@@ -1432,70 +1432,6 @@ int FlowSetProtoEmergencyTimeout(uint8_t proto, uint32_t emerg_new_timeout, uint
     return 1;
 }
 
-/** \brief Set the No Packet Inspection Flag after locking the flow.
- *
- * \param f Flow to set the flag in
- */
-void FlowLockSetNoPacketInspectionFlag(Flow *f) {
-    SCEnter();
-
-    SCLogDebug("flow %p", f);
-    SCMutexLock(&f->m);
-    f->flags |= FLOW_NOPACKET_INSPECTION;
-    SCMutexUnlock(&f->m);
-
-    SCReturn;
-}
-
-/** \brief Set the No Packet Inspection Flag without locking the flow.
- *
- * \param f Flow to set the flag in
- */
-void FlowSetNoPacketInspectionFlag(Flow *f) {
-    SCEnter();
-
-    SCLogDebug("flow %p", f);
-    f->flags |= FLOW_NOPACKET_INSPECTION;
-
-    SCReturn;
-}
-
-/** \brief Set the No payload inspection Flag after locking the flow.
- *
- * \param f Flow to set the flag in
- */
-void FlowLockSetNoPayloadInspectionFlag(Flow *f) {
-    SCEnter();
-
-    SCLogDebug("flow %p", f);
-    SCMutexLock(&f->m);
-    f->flags |= FLOW_NOPAYLOAD_INSPECTION;
-    SCMutexUnlock(&f->m);
-
-    SCReturn;
-}
-
-/** \brief Set the No payload inspection Flag without locking the flow.
- *
- * \param f Flow to set the flag in
- */
-void FlowSetNoPayloadInspectionFlag(Flow *f) {
-    SCEnter();
-
-    SCLogDebug("flow %p", f);
-    f->flags |= FLOW_NOPAYLOAD_INSPECTION;
-
-    SCReturn;
-}
-
-/** \brief set flow flag to disable app layer inspection
- *
- *  \param f *LOCKED* flow
- */
-void FlowSetSessionNoApplayerInspectionFlag(Flow *f) {
-    f->alflags |= FLOW_AL_NO_APPLAYER_INSPECTION;
-}
-
 #ifdef UNITTESTS
 #include "stream-tcp-private.h"
 #include "threads.h"
